@@ -51,18 +51,4 @@ InitPostgresCompat(const char *in_dbname, Oid dboid,
 #endif
 }
 
-static inline void
-get_guc_variables_compat(struct config_generic ***vars, int *num_vars)
-{
-	Assert(vars != NULL);
-	Assert(num_vars != NULL);
-
-#if PG_VERSION_NUM >= 160000
-	*vars = get_guc_variables(num_vars);
-#else
-	*vars = get_guc_variables();
-	*num_vars = GetNumConfigOptions();
-#endif
-}
-
 #endif
